"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.doInit = exports.doEnsure = exports.doGet = void 0;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

const debug = require('debug')('docsmith:configure');

const fs = require('fs-extra');

const read = require('fs').readFileSync;

const path = require('path'); // var npmi = require('npmi');


const omit = require('lodash').omit;

const yaml = require('js-yaml');

global.Promise = require('bluebird');
global.Promise.promisifyAll(fs);

const _require = require('./utils/terminal'),
      promptConfirm = _require.promptConfirm,
      promptRepository = _require.promptRepository,
      exit = _require.exit;

const _require2 = require('./utils/git'),
      doCheck = _require2.doCheck,
      doClone = _require2.doClone,
      doInfo = _require2.doInfo;

const _require3 = require('./workspace'),
      doWorkspaceInit = _require3.doWorkspaceInit,
      doWorkspaceCheck = _require3.doWorkspaceCheck,
      doWorkspacesInfo = _require3.doWorkspacesInfo;

const _require4 = require('./instance'),
      doInstancesInfo = _require4.doInstancesInfo;

const _require5 = require('./package'),
      doPackagesInit = _require5.doPackagesInit,
      doPackagesCheck = _require5.doPackagesCheck;

const _require6 = require('./utils/settings'),
      current = _require6.current,
      settings = _objectWithoutProperties(_require6, ["current"]); // Default to creating global ~/.content and link it to current workspace. (Using this as a workspace detection feature).
// Register workspace in `.content/content.yml` file.
// Allow multi configuration (for development, isolation) by reading the relative `.content` folder.
// `configure`:
//   - checks if workspace. (exists ./.content symlink or folder). if not workspace:
//     - check if global ~/.content. if not ~/.content
//       - `configure/init`
//     - `configure/workspace`
//   - `configure/sync`
// `configure/init`
//   - Ask to confirm for configure global .content
// `configure/workspace`
//   - Ask to confirm for creation of workspace in current folder.
// `configure/sync`
//   - `packages/sync`: Sync package versions.


const toSnakeCase = string => string // Remove all non-word characters
.replace(/[^\w\s]/g, '_') // Replace remaining double spaces for single space
.replace(/\s+/g, ' ') // Add underscores
.toLowerCase().split(' ').join('_');

const doGet = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* () {
    if (!settings.config) return false;
    const git = yield doCheck();
    const repository = yaml.safeLoad(fs.readFileSync(settings.config, 'utf8'));
    debug('repository', repository);
    const instances = yield doInstancesInfo({
      instances: repository.instances || {}
    });
    debug('instances', instances);
    const workspaces = yield doWorkspacesInfo({
      workspaces: repository.workspaces || {}
    });
    debug('workspaces', workspaces);
    return {
      settings,
      repository: _objectSpread({}, repository, {
        instances
      }),
      workspaces,
      git,
      user: toSnakeCase(git.email)
    };
  });

  return function doGet() {
    return _ref.apply(this, arguments);
  };
}();

exports.doGet = doGet;

const doEnsure = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator(function* ({
    non_interactive
  }) {
    let configuration = yield doGet();

    if (!configuration) {
      debug('configuration needs to be initialised');
      configuration = yield doInit({
        non_interactive
      });
    }

    const workspace = yield doWorkspaceCheck({
      non_interactive: true,
      configuration,
      path: process.cwd()
    }); // Check if workspace is properly described in repository.workspace
    //   - If not, just update the workspace ? Or could this be a problem?

    if (!workspace) {
      debug('workspace needs to be initialised');
      configuration = yield doWorkspaceInit({
        non_interactive,
        configuration
      });
    } // Check if packages described in the repository config (possibly updated from the initial bootstrap config)
    // Are present.


    const pkgs = yield doPackagesCheck({
      non_interactive,
      configuration
    });
    debug('pkgs', pkgs[settings.instance]);
    const missing_pkgs = pkgs[settings.instance].filter(x => !x[Object.keys(x)[0]]).map(x => Object.keys(x)[0]);
    debug('missing_pkgs', missing_pkgs);

    if (missing_pkgs.length !== 0) {
      debug('initialising missing packages.');
      configuration = yield doPackagesInit({
        non_interactive,
        configuration,
        packages: missing_pkgs
      });
    }

    return configuration;
  });

  return function doEnsure(_x) {
    return _ref2.apply(this, arguments);
  };
}();

exports.doEnsure = doEnsure;

const doInit = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator(function* ({
    non_interactive,
    configuration
  }) {
    // We check if there are potential conflicts with the current configuration.
    // If there is no key for the instance, then we just initialise with the content package values.
    // If there is a key for the instance we run update.
    let repository;
    let shouldUpgrade = false;

    if (!configuration || !fs.existsSync(configuration.settings.config.replace('/content.yml', ''))) {
      const prompt_repository = yield promptRepository({
        non_interactive,
        default_path: process.env.HOME + '/.content'
      });
      repository = prompt_repository.repository;
      const confirmed = yield promptConfirm({
        non_interactive
      });
      debug('confirmed', confirmed);
      fs.existsSync(repository) || fs.mkdirSync(repository);
      debug('> Content repository: ', repository);
    } else {
      repository = configuration.settings.config.replace('/content.yml', '');
    }

    if (!fs.existsSync(path.join(repository, 'packages'))) {
      const pkgs = path.join(repository, 'packages');
      fs.existsSync(pkgs) || fs.mkdirSync(pkgs);
      debug('> Content packages directory: ', pkgs); // Create or overwrite package.json to enable desired folder structure

      fs.writeFileSync(path.join(pkgs, 'package.json'), JSON.stringify({
        private: true,
        name: 'contentascode',
        version: '0.0.0'
      }), 'utf-8');
      debug('> Content repository package.json created: ', path.join(pkgs, 'package.json'));
    }

    let content;

    if (!fs.existsSync(path.join(repository, 'content.yml'))) {
      debug('No repository configuration');
      content = {
        instances: {
          [settings.instance]: {
            name: settings.description,
            package: settings.package
          }
        }
      };
    } else {
      debug('Existing repository configuration');
      content = yaml.safeLoad(fs.readFileSync(path.join(repository, 'content.yml'), 'utf8'));
      debug('content.yml', content);
      const current = process.cwd();
      debug('current', current);

      if (content && content.instances && content.instances[settings.instance] && content.workspaces && content.workspaces[current]) {
        // TODO: Upgrade path
        debug('Configuration exists and instance already exists. Upgrade needed.');
        shouldUpgrade = true;
      } else if (content.instances === undefined || content.instances[settings.instance] === undefined) {
        debug('Configuration exists but instance does not. Adding instance configuration');
        content = _objectSpread({}, content, {
          instances: _objectSpread({}, content.instances, {
            [settings.instance]: {
              name: settings.description,
              package: settings.package
            }
          })
        });
      } else if (content.workspaces === undefined || content.workspaces[current] === undefined) {
        debug('Configuration exists but workspace does not. Adding workspace configuration');
        content = _objectSpread({}, content, {
          workspaces: _objectSpread({}, content.workspaces, {
            [current]: {}
          })
        });
      } else {
        exit('The content.yml file in ' + path.join(repository, '.content.yml') + ' is not recognised. This beta cannot upgrade configuration yet. Please contact the developer at jun@iilab.org ', new Error('Content.yml structure not recognised'));
      }
    } // const instances = await doInstancesInfo({ instances: content.repository.instances });
    // debug('instances', instances);


    fs.writeFileSync(path.join(repository, 'content.yml'), yaml.safeDump(content), 'utf8');
    debug('> Content repository configuration updated: ', path.join(repository, 'content.yml')); // Now workspace needs to be initialised which for now is done outside `configure` and in `init`.
    // const workspaces = await doWorkspacesInfo({ workspaces: repository.workspaces });

    const instances = yield doInstancesInfo({
      instances: content.instances
    });
    const workspaces = yield doWorkspacesInfo({
      workspaces: content.workspaces || {}
    });
    return {
      settings,
      repository: _objectSpread({}, content, {
        instances
      }),
      workspaces,
      shouldUpgrade
    }; // console.log('content', content);
    // const resClone = await clone(content.packages, content.repository);
    //
    // debug('clone', resClone);
  });

  return function doInit(_x2) {
    return _ref3.apply(this, arguments);
  };
}();

exports.doInit = doInit;

const doUpdate = /*#__PURE__*/function () {
  var _ref4 = _asyncToGenerator(function* ({
    configuration
  }) {
    // Called after an update has been spotted.
    // We need to merge and ask the user for confirmation.
    const configured = yield doPackagesUpgrade({
      non_interactive,
      configuration
    });
  });

  return function doUpdate(_x3) {
    return _ref4.apply(this, arguments);
  };
}(); //
// async function configure({ template, config, link, defaults, verbose }) {
//   // Guard against non implemented feature
//   if (template) {
//     // TODO: Deal with instance templates
//     return exit('\nError while initialising: template instances are not yet supported');
//   }
//
//   // Check git configuration
//   const resCheck = await check();
//   debug('check', resCheck);
//
//   const exists = await fs.pathExists(path.join(config, './content.yml'));
//   if (exists) {
//     const promzard = require('promzard');
//     const file = path.resolve(__dirname, './utils/prompt.js');
//     const content = yaml(read(path.join(config, './content.yml'), 'utf8'));
//
//     console.log(
//       '\n' +
//         '\n' +
//         chalk.grey('============================================================================') +
//         '\n' +
//         chalk.grey('===========                                                      ===========') +
//         '\n' +
//         chalk.grey('===========') +
//         '            ' +
//         pad((settings.description || settings.instance) + ' Initialisation', ' ', 42) +
//         chalk.grey('===========') +
//         '\n' +
//         chalk.grey('===========                                                      ===========') +
//         '\n' +
//         chalk.grey('============================================================================') +
//         '\n' +
//         '\n' +
//         'This script will initialise the current directory as your workspace.' +
//         '\n' +
//         '\n' +
//         'It will also update your global `.content` repository' +
//         '\n' +
//         'and install the following content packages and their dependencies:' +
//         '\n' +
//         '\n' +
//         Object.keys(content.packages).map(repo => '\t' + repo + ': ' + content.packages[repo] + '\n') +
//         '\n' +
//         "Please answer a few questions below and you'll be up and running in no time" +
//         '\n' +
//         '\n' +
//         chalk.yellow(
//           'NOTE: This is a prototype, please accept defaults by hitting enter\n or hit Ctrl-C at any time to exit.'
//         ) +
//         '\n'
//     );
//
//     promzard(file, { content, defaults }, async (err, responses) => {
//       if ((err && err.message == 'canceled') || (responses && responses.confirm !== 'yes')) {
//         console.log('\nExiting without initialising. See ya!');
//         process.exit(0);
//       }
//       if (err) exit('Error while processing prompt results', err);
//
//       fs.existsSync(responses.repository) || fs.mkdirSync(responses.repository);
//       debug('> Content repository: ', responses.repository);
//
//       const pkgs = path.join(responses.repository, 'packages');
//       fs.existsSync(pkgs) || fs.mkdirSync(pkgs);
//       debug('> Content packages directory: ', pkgs);
//
//       // Change working directory temporarily as npm api is insufficient.
//       const current = process.cwd();
//       try {
//         process.chdir(pkgs);
//         debug('changed directory: ', pkgs);
//       } catch (err) {
//         exit('\nError while changing directory', err);
//       }
//
//       // Create or overwrite package.json to enable desired folder structure
//       fs.writeFileSync(
//         path.join(process.cwd(), 'package.json'),
//         JSON.stringify({ private: true, name: 'contentascode', version: '0.0.0' }),
//         'utf-8'
//       );
//       debug('> Content repository configuration created: ', path.join(process.cwd(), 'package.json'));
//
//       console.log('content', content);
//       const resClone = await clone(content.packages, content.repository);
//
//       debug('clone', resClone);
//     });
//   } else {
//     console.error(
//       'Error: CLI tool does not have a content.yml configuration file. Please report the error to the developer.'
//     );
//     process.exit(1, '');
//   }
// }
//# sourceMappingURL=configure.js.map