"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.doInstancesInfo = void 0;

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const debug = require('debug')('docsmith:instance');

const fs = require('fs-extra');

const path = require('path');

const yaml = require('js-yaml').safeLoad;

const read = require('fs').readFileSync;

global.Promise = require('bluebird');
global.Promise.promisifyAll(fs);

const _require = require('./utils/terminal'),
      exit = _require.exit;

const _require2 = require('./utils/git'),
      doInfo = _require2.doInfo;

const _require3 = require('./utils/npm'),
      doListGlobal = _require3.doListGlobal;

const isLink = path => {
  try {
    const lstat = fs.lstatSync(path);
    debug('lstat', lstat);
    debug('lstat.isSymbolicLink()', lstat.isSymbolicLink());
    return lstat.isSymbolicLink();
  } catch (e) {
    if (e.code == 'ENOENT') {
      return e;
    }

    process.exit();
  }
};

const toDir = path => isLink(path) ? fs.readlinkSync(path) : path;

const doInstancesInfo = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* ({
    instances
  }, s) {
    const settings = s || require('./utils/settings').current();

    settings.package = settings.pkg;
    if (!instances) exit('Instances configuration missing. Your ~/.content/content.yml file might be corrupted. Try deleting it and run ' + settings.instance + ' init.', Error('instances configuration not found'));
    debug('instances', instances);
    return Promise.all(Object.keys(instances).map( /*#__PURE__*/function () {
      var _ref2 = _asyncToGenerator(function* (instance) {
        debug('pkg', instances[instance].package);

        const _ref3 = yield doListGlobal(instances[instance].package),
              pth = _ref3.path,
              link = _ref3.link,
              version = _ref3.version,
              name = _ref3.name,
              description = _ref3.description;

        debug('path', pth);
        debug('link', link); // Gather the content of the content.yml.

        const content = yaml(read(path.join(pth, './content.yml'), 'utf8'));
        debug('settings.packages', settings.packages); // Lookup package content.yml

        const pkgs = Object.keys(content.packages).map(pkg => ({
          [pkg]: _objectSpread({
            package: content.packages[pkg]
          }, settings.packages && fs.existsSync(path.join(settings.packages, pkg, './content.yml')) ? _objectSpread({
            status: 'installed',
            install: toDir(path.join(settings.packages, pkg))
          }, yaml(read(path.join(settings.packages, pkg, './content.yml'), 'utf8'))) : {
            status: 'uninstalled'
          })
        })).reduce((acc, cur) => _objectSpread({}, acc, {}, cur), {});
        debug('pkgs', pkgs);
        return {
          instance,
          package: instances[instance].package,
          name: description,
          version,
          path: pth,
          link,
          content: _objectSpread({}, content, {
            packages: pkgs
          })
        };
      });

      return function (_x3) {
        return _ref2.apply(this, arguments);
      };
    }())).then(pkgs => pkgs.reduce((acc, _ref4) => {
      let instance = _ref4.instance,
          cur = _objectWithoutProperties(_ref4, ["instance"]);

      return _objectSpread({}, acc, {
        [instance]: cur
      });
    }, {})); // .then(see => console.log('see', see));
  });

  return function doInstancesInfo(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

exports.doInstancesInfo = doInstancesInfo;
//# sourceMappingURL=instance.js.map