"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.doInfo = exports.doPull = exports.doClone = exports.doCheck = void 0;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const debug = require('debug')('docsmith:git');

const nodegit = require('nodegit');

const _require = require('./npm'),
      taskView = _require.taskView;

const ProgressBar = require('progress');

const _require2 = require('folktale/concurrency/task'),
      task = _require2.task,
      waitAll = _require2.waitAll,
      of = _require2.of,
      fromPromised = _require2.fromPromised;

const Validation = require('folktale/validation');

const Success = Validation.Success,
      Failure = Validation.Failure;

const _require3 = require('./terminal'),
      exit = _require3.exit;

const collect = validations => validations.reduce((a, b) => a.concat(b), Success());

const hello = task(resolver => resolver.resolve('hello'));

const id = x => x;

const delay = ms => task(resolver => {
  const timerId = setTimeout(() => resolver.resolve(ms), ms);
  resolver.cleanup(() => {
    clearTimeout(timerId);
  });
}); // Key -> Task Validation Error ()


const isConfigValueSet = fromPromised(key => nodegit.Config.openDefault().then(config => config.getStringBuf(key)).then(buf => buf ? Success(buf) : Failure(`Git configuration key ${key} is not set\n`)).catch(e => Failure(e))); // Task [ Validation Error () ]

const requirements = waitAll([isConfigValueSet('user.email'), isConfigValueSet('user.name')]);

const doCheck = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* () {
    debug('Checking requirements'); // console.log('checkUserEmail[a,b]', checkUserEmail(['a', undefined]));

    const res = yield requirements // Task [ Validation Error () ] -> Task Validation [ Error ] ()
    .map(([email, name]) => Success(email => name => ({
      email,
      name
    })).apply(email).apply(name)).run().promise();
    return res.matchWith({
      Success: ({
        value
      }) => _objectSpread({}, value),
      Failure: () => {
        console.log('Please check that git is installed properly.');
        process.exit();
      }
    });
  });

  return function doCheck() {
    return _ref.apply(this, arguments);
  };
}(); // Key -> Task Validation Error ()


exports.doCheck = doCheck;
const infoRepo = fromPromised(({
  path
}) => {
  return nodegit.Repository.open(path).then(repository => Promise.all([repository.getBranchCommit('master'), repository.getCurrentBranch().then(ref => ref.toString()), repository.getReferences(nodegit.Reference.TYPE.LISTALL).then(refs => refs.filter(ref => ref.isBranch()).map(ref => ref.toString()))])).then(([commit, current, branches]) => Success({
    commit: commit.sha(),
    current,
    branches
  })).catch(e => Failure(e));
});

const doInfo = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator(function* ({
    path
  }) {
    debug('Getting git info');
    const res = yield infoRepo({
      path
    }) // Task [ Validation Error () ] -> Task Validation [ Error ] ()
    // .map(([email, name]) =>
    //   Success(email => name => ({ email, name }))
    //     .apply(email)
    //     .apply(name)
    // )
    .run().promise();
    return res.matchWith({
      Success: ({
        value
      }) => _objectSpread({}, value),
      Failure: ({
        value
      }) => {
        console.log(`Please check that git is installed properly. Error: ${value}`);
        process.exit();
      }
    });
  });

  return function doInfo(_x) {
    return _ref2.apply(this, arguments);
  };
}(); // `docsmith-init`
//   - pre-conditions
//     - If folder exists,
//       - If not git repo: ask for upgrade confirmation
//     - If folder doesn't exist:
//       - `init`
//         - cloneRepo
// `load`
//   - If folder doesn't exist: Ask to `init`
//   - If folder exists:
//     - If not git repo: ask to `init`
//     - If git repo: fetch, what to do if there are commits since a release/tag?
//     - If new tag (persist in .content)
//       - Ask for content update confirmation.
//         - Show updates and conflict visually?
//         - Try to rebase local branch onto it.
//       - What if not updated?
//         - When branch is pushed, possibly push resolution of merge conflicts to editor.
//         - Editor will create an integration branch where they will rebase and solve conflicts.
// Key -> Task Validation Error ()


exports.doInfo = doInfo;
const cloneRepo = fromPromised(({
  url,
  name
}) => {
  // const options = new nodegit.CloneOptions();
  // const fetchOpts = new nodegit.FetchOptions();
  // options.fetchOpts = fetchOpts;
  // const callbacks = new nodegit.RemoteCallbacks();
  // fetchOpts.callbacks = callbacks;
  let bar;
  const newObjects = 0;
  debug('Cloning');
  const opts = {
    fetchOpts: {
      callbacks: {
        transferProgress(progress) {
          bar = bar || new ProgressBar('Cloning [:bar] :percent :etas :url ', {
            complete: '=',
            incomplete: ' ',
            width: 100,
            total: 100
          });
          bar.update(progress.receivedObjects() / progress.totalObjects(), {
            url
          });
        }

      }
    }
  }; // callbacks.transferProgress = function(stats) {
  //   debug('Progress');
  //   const progress = 100 * (stats.receivedObjects() + stats.indexedObjects()) / (stats.totalObjects() * 2);
  //   return progress;
  // };

  return nodegit.Clone(url, name, opts).then(repository => {
    bar.update(1);
    return repository;
  }).then(repository => repository.getBranchCommit('master')).then(commit => Success(commit.sha())).catch(e => Failure(e));
}); // Key -> Task Validation Error ()

const pullRepo = fromPromised(path => {
  // const options = new nodegit.CloneOptions();
  // const fetchOpts = new nodegit.FetchOptions();
  // options.fetchOpts = fetchOpts;
  // const callbacks = new nodegit.RemoteCallbacks();
  // fetchOpts.callbacks = callbacks;
  let bar;
  const newObjects = 0;
  debug('Pulling', path);
  return nodegit.Repository.open(path).then(repository => {
    repository.fetchAll();
    return repository;
  }).then(repository => repository.mergeBranches('master', 'origin/master')).then(oid => Success(oid)).catch(e => Failure(e));
}); // [ gitURL ] -> Task [ Validation Error () ]

const doCloneRepos = (packages, path) => waitAll(packages.map(repo => cloneRepo(repo, path))); // [ gitURL ] -> Task [ Validation Error () ]


const doPullRepos = paths => waitAll(paths.map(path => pullRepo(path))); // Info -> gitURL


const getRepo = ({
  value: info
}) => Object.keys(info).map(key => ({
  url: info[key].repository.url.replace('git+https', 'https'),
  name: info[key].name
})); // objPck: the packages key of the content.yml file of the instance
// path: the destination path for cloning the repo.
// objPck -> path -> Task Validation [ CommitSha ]


const doClone = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator(function* (packages, path) {
    debug('Getting git url from npm'); // debug('objPck', objPck);
    // const packages = Object.keys(objPck); //.map(key => objPck[key].split('@')[0]);

    debug('clone.packages', packages); // [ Package ] -> Task [ Validation Error Info ]

    const res = yield taskView(packages.map(pkg => pkg.split('@')[0])) // Task Validation Error [ Info ] -> Task Validation Error [ Repos ]
    .map(getRepo) // Task Validation Error [ gitURL ] -> Task [ Validation Error () ]
    .chain(repos => doCloneRepos(repos, path)) // Task [ Validation Error () ] -> Task Validation [ Error ] ()
    .map(collect).run().promise();
    return res.matchWith({
      Success: ({
        value
      }) => ({
        value
      }),
      Failure: ({
        value
      }) => {
        exit(`Error while cloning ${Object.keys(packages)} into ${path}`, value);
      }
    });
  });

  return function doClone(_x2, _x3) {
    return _ref3.apply(this, arguments);
  };
}(); // objPck: the packages key of the content.yml file of the instance
// path: the destination path for cloning the repo.
// objPck -> path -> Task Validation [ CommitSha ]


exports.doClone = doClone;

const doPull = /*#__PURE__*/function () {
  var _ref4 = _asyncToGenerator(function* (objPck) {
    debug('Getting git url from npm');
    debug('objPck', objPck);
    const packages = Object.keys(objPck).map(key => objPck[key].install); //.map(key => objPck[key].split('@')[0]);

    debug('pull.packages', packages); // [ path ] -> Task [ Validation Error [ path ] ]

    const res = yield of(packages) // Task Validation Error [ path ] -> Task [ Validation Error () ]
    .chain(paths => doPullRepos(paths)) // Task [ Validation Error () ] -> Task Validation [ Error ] ()
    .map(collect).run().promise();
    return res.matchWith({
      Success: ({
        value
      }) => ({
        value
      }),
      Failure: ({
        value
      }) => {
        exit(`Error while pulling ${Object.keys(objPck)}`, value);
      }
    });
  });

  return function doPull(_x4) {
    return _ref4.apply(this, arguments);
  };
}();

exports.doPull = doPull;
//# sourceMappingURL=git.js.map