"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.exit = exports.promptNew = exports.promptWorkspace = exports.promptRepository = exports.promptConfirm = exports.banner = void 0;

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const debug = require('debug')('docsmith:terminal');

const chalk = require('chalk');

const promzard = require('promzard');

global.Promise = require('bluebird');
const prompt = global.Promise.promisify(promzard);

const path = require('path');

const settings = require('./settings');

const pad = (string, char, length) => (string + char.repeat(length)).slice(0, length);

const banner = ({
  title,
  message
}) => {
  console.log('\n' + '\n' + chalk.grey('============================================================================') + '\n' + chalk.grey('===========                                                      ===========') + '\n' + chalk.grey('===========') + '            ' + pad((settings.description || settings.instance) + ' ' + title, ' ', 42) + chalk.grey('===========') + '\n' + chalk.grey('===========                                                      ===========') + '\n' + chalk.grey('============================================================================') + '\n' + '\n' + message + '\n' + chalk.yellow('Hit Ctrl-C at any time to exit.') + '\n');
};

exports.banner = banner;

const promptConfirm = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* ({
    non_interactive
  }) {
    debug('non_interactive', non_interactive);
    const confirmation = path.resolve(__dirname, './prompts/confirmation.js');
    debug('confirmation', confirmation);

    try {
      const _ref2 = yield prompt(confirmation, {
        non_interactive
      }),
            confirmed = _ref2.confirm;

      debug('confirmed', confirmed);

      if (!confirmed) {
        exit("User didn't confirm. Exiting.");
      } else {
        return confirmed;
      }
    } catch (e) {
      if (e.message === 'canceled') {
        exit('User canceled. Exiting.');
      } else {
        throw e;
      }
    }
  });

  return function promptConfirm(_x) {
    return _ref.apply(this, arguments);
  };
}();

exports.promptConfirm = promptConfirm;

const promptRepository = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator(function* ({
    non_interactive,
    default_path
  }) {
    debug('non_interactive', non_interactive);
    const content_repository = path.resolve(__dirname, './prompts/content_repository.js');
    debug('content_repository', content_repository);

    try {
      const _ref4 = yield prompt(content_repository, {
        non_interactive,
        default_path
      }),
            repository = _ref4.repository;

      return {
        repository
      };
    } catch (e) {
      if (e.message === 'canceled') {
        exit('User canceled. Exiting.');
      } else {
        throw e;
      }
    }
  });

  return function promptRepository(_x2) {
    return _ref3.apply(this, arguments);
  };
}();

exports.promptRepository = promptRepository;

const promptWorkspace = /*#__PURE__*/function () {
  var _ref5 = _asyncToGenerator(function* ({
    non_interactive,
    default_path
  }) {
    debug('non_interactive', non_interactive);
    const workspace = path.resolve(__dirname, './prompts/workspace.js');
    debug('workspace', workspace);

    try {
      const _ref6 = yield prompt(workspace, {
        non_interactive,
        default_path
      }),
            confirm = _ref6.confirm;

      return {
        confirm
      };
    } catch (e) {
      if (e.message === 'canceled') {
        exit('User canceled. Exiting.');
      } else {
        throw e;
      }
    }
  });

  return function promptWorkspace(_x3) {
    return _ref5.apply(this, arguments);
  };
}();

exports.promptWorkspace = promptWorkspace;

const promptNew = /*#__PURE__*/function () {
  var _ref7 = _asyncToGenerator(function* ({
    non_interactive,
    default_name,
    default_template,
    templates
  }) {
    debug('non_interactive', non_interactive);
    const creation = path.resolve(__dirname, './prompts/creation.js');
    debug('creation', creation);

    try {
      const _ref8 = yield prompt(creation, {
        non_interactive,
        default_name,
        default_template,
        templates
      }),
            name = _ref8.name,
            template = _ref8.template;

      debug('name', name);
      debug('template', template);
      return {
        name,
        template
      };
    } catch (e) {
      if (e.message === 'canceled') {
        exit('User canceled. Exiting.');
      } else {
        throw e;
      }
    }
  });

  return function promptNew(_x4) {
    return _ref7.apply(this, arguments);
  };
}();

exports.promptNew = promptNew;

const exit = (message, error) => {
  {
    if (error) {
      console.log('error', error);
      console.log(chalk.red('\n' + message + '\n'));
      console.log(chalk.grey('\n=============================================================================\n\n') + chalk.red('Please alert the developer by submitting an issue \nat https://github.com/docsmith/docsmith/issues and copy the whole output of the command above.\n\nApologies for the inconvenience!\n'));
      process.exit(1);
    } else {
      console.log(chalk.grey('\n'));
      console.log(chalk.grey('\n' + message + '\n'));
      console.log(chalk.grey('\n============================================================================\n\n'));
      process.exit(0);
    }
  }
};

exports.exit = exit;
//# sourceMappingURL=terminal.js.map