"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.doWorkspacesInfo = exports.doWorkspaceUpgrade = exports.doWorkspaceCheck = exports.doWorkspaceInit = void 0;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const debug = require('debug')('docsmith:workspace');

const fs = require('fs-extra');

const path = require('path');

global.Promise = require('bluebird');
global.Promise.promisifyAll(fs);

const _require = require('./utils/terminal'),
      exit = _require.exit;

const _require2 = require('./package'),
      doPackagesInit = _require2.doPackagesInit,
      doPackagesUpgrade = _require2.doPackagesUpgrade;

const _require3 = require('./utils/git'),
      doInfo = _require3.doInfo;

const doWorkspacesInfo = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* ({
    workspaces
  }) {
    return Promise.all(Object.keys(workspaces).map( /*#__PURE__*/function () {
      var _ref2 = _asyncToGenerator(function* (workspace) {
        const files = yield fs.readdirAsync(workspace);
        const instances = files.filter(file => file[0] === '@' && !/\.bak/.test(file));
        debug('instances', instances);
        return Promise.all(instances.map( /*#__PURE__*/function () {
          var _ref3 = _asyncToGenerator(function* (instance) {
            try {
              const folders = yield fs.readdirAsync(path.join(workspace, instance));
              const links = folders.filter(link => link[0] !== '.');
              debug('links', links);
              return Promise.all(links.map( /*#__PURE__*/function () {
                var _ref4 = _asyncToGenerator(function* (lnk_path) {
                  const lnk = yield fs.readlinkAsync(path.join(workspace, instance, lnk_path));
                  debug('lnk', lnk);
                  const link = path.resolve(workspace, lnk);
                  debug('link', link);
                  const pkg_name = link.split('/packages/')[1].split('/')[0];
                  const pkg_path = path.join(link.split('/packages/')[0], 'packages', pkg_name);

                  const pkg = require(path.join(pkg_path, 'package.json'));

                  const name = pkg.name;
                  const version = pkg.version;

                  const _ref5 = yield doInfo({
                    path: pkg_path
                  }),
                        commit = _ref5.commit,
                        current = _ref5.current,
                        branches = _ref5.branches;

                  return {
                    [instance + '/' + lnk_path]: {
                      source: link,
                      package: name,
                      version,
                      current,
                      branches,
                      commit
                    }
                  };
                });

                return function (_x4) {
                  return _ref4.apply(this, arguments);
                };
              }())).then(link => ({
                [instance]: link.reduce((acc, cur) => _objectSpread({}, acc, {}, cur), {})
              }));
            } catch (e) {
              if (e.code === 'ENOENT') exit('Could not find the folder ' + path.join(workspace, instance), e);else throw e;
            }
          });

          return function (_x3) {
            return _ref3.apply(this, arguments);
          };
        }())).then(instance => ({
          [workspace]: instance.reduce((acc, cur) => _objectSpread({}, acc, {}, cur), {})
        }));
      });

      return function (_x2) {
        return _ref2.apply(this, arguments);
      };
    }())).then(workspaces => workspaces.reduce((acc, cur) => _objectSpread({}, acc, {}, cur), {}));
  });

  return function doWorkspacesInfo(_x) {
    return _ref.apply(this, arguments);
  };
}();

exports.doWorkspacesInfo = doWorkspacesInfo;

const doWorkspaceCheck = /*#__PURE__*/function () {
  var _ref6 = _asyncToGenerator(function* ({
    non_interactive,
    configuration
  }) {
    // Check if current folder is workspace.
    const current = process.cwd();
    debug('Object.keys(configuration.workspaces)', Object.keys(configuration.workspaces).map(key => Object.keys(configuration.workspaces[key])[0]));
    const known = configuration.workspaces && Object.keys(configuration.workspaces).map(key => Object.keys(configuration.workspaces[key])[0]).includes('@' + configuration.settings.instance);
    debug('known', known);

    if (known) {
      return current;
    } else if (non_interactive) {
      return false;
    }

    exit('Please run this command within a workspace. Known workspaces are:\n' + Object.keys(configuration.workspaces).map(key => ' - ' + Object.keys(configuration.workspaces[key])[0]).join('\n'));
    return false;
  });

  return function doWorkspaceCheck(_x5) {
    return _ref6.apply(this, arguments);
  };
}();

exports.doWorkspaceCheck = doWorkspaceCheck;

const doWorkspaceInit = /*#__PURE__*/function () {
  var _ref7 = _asyncToGenerator(function* ({
    non_interactive,
    configuration
  }) {
    // Change working directory temporarily as npm api is insufficient.
    configuration.settings = require('./utils/settings').current();
    configuration.settings.package = configuration.settings.pkg;
    debug('configuration', configuration);
    const current = process.cwd();

    try {
      process.chdir(configuration.settings.packages);
      debug('changed directory: ', configuration.settings.packages);
    } catch (err) {
      exit('\nError while changing directory', err);
    } // Create folder for @instance


    if (configuration.workspaces && configuration.workspaces[current] && configuration.workspaces[current]['@' + configuration.settings.instance] !== undefined) {
      // TODO: Do a packages update instead.
      exit('The workspace has already been initialised. Please use load or new to create a new content package.');
    } else {
      const instanceWorkspacePath = path.join(current, '@' + configuration.settings.instance);
      fs.mkdirSync(instanceWorkspacePath);
      debug('Created instance folder in workspace', instanceWorkspacePath);
    } // Get packages for instance.


    const packages = configuration.repository.instances[configuration.settings.instance].content.packages;
    debug('packages', packages); // Initialise and symlink packages

    const init = yield doPackagesInit({
      non_interactive,
      configuration,
      packages,
      current
    });
    debug('init', init); // restore working directory

    try {
      process.chdir(current);
      debug('changed directory: ', current);
    } catch (err) {
      exit('\nError while changing directory', err);
    }

    return configuration;
  });

  return function doWorkspaceInit(_x6) {
    return _ref7.apply(this, arguments);
  };
}();

exports.doWorkspaceInit = doWorkspaceInit;

const doWorkspaceUpgrade = /*#__PURE__*/function () {
  var _ref8 = _asyncToGenerator(function* ({
    non_interactive,
    configuration
  }) {
    // Change working directory temporarily as npm api is insufficient.
    configuration.settings = require('./utils/settings').current();
    configuration.settings.package = configuration.settings.pkg;
    debug('configuration', configuration);
    const current = process.cwd();

    try {
      process.chdir(configuration.settings.packages);
      debug('changed directory: ', configuration.settings.packages);
    } catch (err) {
      exit('\nError while changing directory', err);
    } // Get packages for instance.


    const packages = configuration.repository.instances[configuration.settings.instance].content.packages;
    debug('packages', packages); // Initialise and symlink packages

    const upgrade = yield doPackagesUpgrade({
      non_interactive,
      configuration,
      packages,
      current
    });
    debug('upgrade', upgrade); // restore working directory

    try {
      process.chdir(current);
      debug('changed directory: ', current);
    } catch (err) {
      exit('\nError while changing directory', err);
    }

    return configuration;
  });

  return function doWorkspaceUpgrade(_x7) {
    return _ref8.apply(this, arguments);
  };
}();

exports.doWorkspaceUpgrade = doWorkspaceUpgrade;
//# sourceMappingURL=workspace.js.map